@@ -12,7 +12,7 @@ mod x509;
 
 use std::{borrow::Cow, fmt::Debug, str::FromStr};
 
-use hmac::{Mac, NewMac};
+use hmac::{digest::KeyInit, Mac};
 use rand::Rng;
 use serde::Deserialize;
 use typed_builder::TypedBuilder;
@@ -68,7 +68,7 @@ pub enum AuthMechanism {
     Gssapi,
 
     /// The SASL PLAIN mechanism, as defined in [RFC 4616](), is used in MongoDB to perform LDAP
-    /// authentication and cannot be used for any other type of authenticaiton.
+    /// authentication and cannot be used for any other type of authentication.
     /// Since the credentials are stored outside of MongoDB, the "$external" database must be used
     /// for authentication.
     ///
@@ -352,7 +352,7 @@ pub struct Credential {
 }
 
 impl Credential {
-    #[cfg(all(test, not(feature = "sync")))]
+    #[cfg(all(test, not(feature = "sync"), not(feature = "tokio-sync")))]
     pub(crate) fn into_document(mut self) -> Document {
         use crate::bson::Bson;
 
@@ -383,7 +383,7 @@ impl Credential {
     }
 
     /// If the mechanism is missing, append the appropriate mechanism negotiation key-value-pair to
-    /// the provided isMaster command document.
+    /// the provided hello or legacy hello command document.
     pub(crate) fn append_needed_mechanism_negotiation(&self, command: &mut Document) {
         if let (Some(username), None) = (self.username.as_ref(), self.mechanism.as_ref()) {
             command.insert(
@@ -515,13 +515,13 @@ pub(crate) fn generate_nonce() -> String {
     base64::encode(&result)
 }
 
-fn mac<M: Mac + NewMac>(
+fn mac<M: Mac + KeyInit>(
     key: &[u8],
     input: &[u8],
     auth_mechanism: &str,
 ) -> Result<impl AsRef<[u8]>> {
-    let mut mac =
-        M::new_from_slice(key).map_err(|_| Error::unknown_authentication_error(auth_mechanism))?;
+    let mut mac = <M as Mac>::new_from_slice(key)
+        .map_err(|_| Error::unknown_authentication_error(auth_mechanism))?;
     mac.update(input);
     Ok(mac.finalize().into_bytes())
 }
