@@ -14,13 +14,14 @@ use uuid::Uuid;
 
 use crate::{
     bson::{doc, spec::BinarySubtype, Binary, Bson, Document, Timestamp},
+    cmap::conn::PinnedConnectionHandle,
     error::{ErrorKind, Result},
     operation::{AbortTransaction, CommitTransaction, Operation},
     options::{SessionOptions, TransactionOptions},
+    runtime,
     sdam::{ServerInfo, TransactionSupportStatus},
     selection_criteria::SelectionCriteria,
     Client,
-    RUNTIME,
 };
 pub use cluster_time::ClusterTime;
 pub(super) use pool::ServerSessionPool;
@@ -108,13 +109,14 @@ pub struct ClientSession {
     options: Option<SessionOptions>,
     pub(crate) transaction: Transaction,
     pub(crate) snapshot_time: Option<Timestamp>,
+    pub(crate) operation_time: Option<Timestamp>,
 }
 
-#[derive(Clone, Debug)]
+#[derive(Debug)]
 pub(crate) struct Transaction {
     pub(crate) state: TransactionState,
     pub(crate) options: Option<TransactionOptions>,
-    pub(crate) pinned_mongos: Option<SelectionCriteria>,
+    pub(crate) pinned: Option<TransactionPin>,
     pub(crate) recovery_token: Option<Document>,
 }
 
@@ -132,15 +134,38 @@ impl Transaction {
     pub(crate) fn abort(&mut self) {
         self.state = TransactionState::Aborted;
         self.options = None;
-        self.pinned_mongos = None;
+        self.pinned = None;
     }
 
     pub(crate) fn reset(&mut self) {
         self.state = TransactionState::None;
         self.options = None;
-        self.pinned_mongos = None;
+        self.pinned = None;
         self.recovery_token = None;
     }
+
+    pub(crate) fn pinned_mongos(&self) -> Option<&SelectionCriteria> {
+        match &self.pinned {
+            Some(TransactionPin::Mongos(s)) => Some(s),
+            _ => None,
+        }
+    }
+
+    pub(crate) fn pinned_connection(&self) -> Option<&PinnedConnectionHandle> {
+        match &self.pinned {
+            Some(TransactionPin::Connection(c)) => Some(c),
+            _ => None,
+        }
+    }
+
+    fn take(&mut self) -> Self {
+        Transaction {
+            state: self.state.clone(),
+            options: self.options.take(),
+            pinned: self.pinned.take(),
+            recovery_token: self.recovery_token.take(),
+        }
+    }
 }
 
 impl Default for Transaction {
@@ -148,7 +173,7 @@ impl Default for Transaction {
         Self {
             state: TransactionState::None,
             options: None,
-            pinned_mongos: None,
+            pinned: None,
             recovery_token: None,
         }
     }
@@ -168,6 +193,12 @@ pub(crate) enum TransactionState {
     Aborted,
 }
 
+#[derive(Debug)]
+pub(crate) enum TransactionPin {
+    Mongos(SelectionCriteria),
+    Connection(PinnedConnectionHandle),
+}
+
 impl ClientSession {
     /// Creates a new `ClientSession` wrapping the provided server session.
     pub(crate) fn new(
@@ -184,6 +215,7 @@ impl ClientSession {
             options,
             transaction: Default::default(),
             snapshot_time: None,
+            operation_time: None,
         }
     }
 
@@ -227,6 +259,21 @@ impl ClientSession {
         }
     }
 
+    /// Advance operation time for this session. If the provided timestamp is earlier than this
+    /// session's current operation time, then the operation time is unchanged.
+    pub fn advance_operation_time(&mut self, ts: Timestamp) {
+        self.operation_time = match self.operation_time {
+            Some(current_op_time) if current_op_time < ts => Some(ts),
+            None => Some(ts),
+            _ => self.operation_time,
+        }
+    }
+
+    /// The operation time returned by the last operation executed in this session.
+    pub fn operation_time(&self) -> Option<Timestamp> {
+        self.operation_time
+    }
+
     /// Mark this session (and the underlying server session) as dirty.
     pub(crate) fn mark_dirty(&mut self) {
         self.server_session.dirty = true;
@@ -256,13 +303,18 @@ impl ClientSession {
 
     /// Pin mongos to session.
     pub(crate) fn pin_mongos(&mut self, address: ServerAddress) {
-        self.transaction.pinned_mongos = Some(SelectionCriteria::Predicate(Arc::new(
-            move |server_info: &ServerInfo| *server_info.address() == address,
+        self.transaction.pinned = Some(TransactionPin::Mongos(SelectionCriteria::Predicate(
+            Arc::new(move |server_info: &ServerInfo| *server_info.address() == address),
         )));
     }
 
-    pub(crate) fn unpin_mongos(&mut self) {
-        self.transaction.pinned_mongos = None;
+    /// Pin the connection to the session.
+    pub(crate) fn pin_connection(&mut self, handle: PinnedConnectionHandle) {
+        self.transaction.pinned = Some(TransactionPin::Connection(handle));
+    }
+
+    pub(crate) fn unpin(&mut self) {
+        self.transaction.pinned = None;
     }
 
     /// Whether this session is dirty.
@@ -319,7 +371,7 @@ impl ClientSession {
                 .into());
             }
             TransactionState::Committed { .. } => {
-                self.unpin_mongos(); // Unpin session if previous transaction is committed.
+                self.unpin(); // Unpin session if previous transaction is committed.
             }
             _ => {}
         }
@@ -495,8 +547,8 @@ impl ClientSession {
                     .as_ref()
                     .and_then(|options| options.write_concern.as_ref())
                     .cloned();
-                let selection_criteria = self.transaction.pinned_mongos.clone();
-                let abort_transaction = AbortTransaction::new(write_concern, selection_criteria);
+                let abort_transaction =
+                    AbortTransaction::new(write_concern, self.transaction.pinned.take());
                 self.transaction.abort();
                 // Errors returned from running an abortTransaction command should be ignored.
                 let _result = self
@@ -524,6 +576,7 @@ struct DroppedClientSession {
     options: Option<SessionOptions>,
     transaction: Transaction,
     snapshot_time: Option<Timestamp>,
+    operation_time: Option<Timestamp>,
 }
 
 impl From<DroppedClientSession> for ClientSession {
@@ -536,6 +589,7 @@ impl From<DroppedClientSession> for ClientSession {
             options: dropped_session.options,
             transaction: dropped_session.transaction,
             snapshot_time: dropped_session.snapshot_time,
+            operation_time: dropped_session.operation_time,
         }
     }
 }
@@ -549,17 +603,18 @@ impl Drop for ClientSession {
                 client: self.client.clone(),
                 is_implicit: self.is_implicit,
                 options: self.options.clone(),
-                transaction: self.transaction.clone(),
+                transaction: self.transaction.take(),
                 snapshot_time: self.snapshot_time,
+                operation_time: self.operation_time,
             };
-            RUNTIME.execute(async move {
+            runtime::execute(async move {
                 let mut session: ClientSession = dropped_session.into();
                 let _result = session.abort_transaction().await;
             });
         } else {
             let client = self.client.clone();
             let server_session = self.server_session.clone();
-            RUNTIME.execute(async move {
+            runtime::execute(async move {
                 client.check_in_server_session(server_session).await;
             });
         }
