@@ -16,18 +16,21 @@ use crate::{
     error::{Error, ErrorKind, Result},
     event::cmap::ConnectionPoolOptions as EventOptions,
     options::TlsOptions,
+    runtime,
     runtime::AsyncJoinHandle,
     sdam::{ServerUpdate, ServerUpdateSender},
     test::{
         assert_matches,
+        eq_matches,
+        log_uncaptured,
         run_spec_test,
         EventClient,
+        MatchErrExt,
         Matchable,
         CLIENT_OPTIONS,
         LOCK,
         SERVER_API,
     },
-    RUNTIME,
 };
 use bson::doc;
 
@@ -37,6 +40,8 @@ const TEST_DESCRIPTIONS_TO_SKIP: &[&str] = &[
     // WaitQueueTimeoutMS is not supported
     "must aggressively timeout threads enqueued longer than waitQueueTimeoutMS",
     "waiting on maxConnecting is limited by WaitQueueTimeoutMS",
+    // TODO DRIVERS-1785 remove this skip when test event order is fixed
+    "error during minPoolSize population clears pool",
 ];
 
 /// Many different types of CMAP events are emitted from tasks spawned in the drop
@@ -96,14 +101,12 @@ struct CmapThread {
 impl CmapThread {
     fn start(state: Arc<State>) -> Self {
         let (sender, mut receiver) = tokio::sync::mpsc::unbounded_channel::<Operation>();
-        let handle = RUNTIME
-            .spawn(async move {
-                while let Some(operation) = receiver.recv().await {
-                    operation.execute(state.clone()).await?;
-                }
-                Ok(())
-            })
-            .unwrap();
+        let handle = runtime::spawn(async move {
+            while let Some(operation) = receiver.recv().await {
+                operation.execute(state.clone()).await?;
+            }
+            Ok(())
+        });
 
         Self {
             dispatcher: sender,
@@ -123,7 +126,7 @@ impl Executor {
         let handler = Arc::new(EventHandler::new());
         let error = test_file.error;
 
-        let mut pool_options = test_file.pool_options.unwrap_or_else(Default::default);
+        let mut pool_options = test_file.pool_options.unwrap_or_default();
         pool_options.tls_options = CLIENT_OPTIONS.tls_options();
         pool_options.cmap_event_handler = Some(handler.clone());
         pool_options.server_api = SERVER_API.clone();
@@ -161,11 +164,14 @@ impl Executor {
 
         // Mock a monitoring task responding to errors reported by the pool.
         let manager = pool.manager.clone();
-        RUNTIME.execute(async move {
+        runtime::execute(async move {
             while let Some(update) = update_receiver.recv().await {
-                match update.into_message() {
-                    ServerUpdate::Error { error, .. } => manager.clear(error.cause, None).await,
+                match update.message() {
+                    ServerUpdate::Error { error, .. } => {
+                        manager.clear(error.cause.clone(), None).await
+                    }
                 }
+                drop(update);
             }
         });
 
@@ -218,7 +224,7 @@ impl Operation {
     /// Execute this operation.
     async fn execute(self, state: Arc<State>) -> Result<()> {
         match self {
-            Operation::Wait { ms } => RUNTIME.delay_for(Duration::from_millis(ms)).await,
+            Operation::Wait { ms } => runtime::delay_for(Duration::from_millis(ms)).await,
             Operation::WaitForThread { target } => {
                 state
                     .threads
@@ -237,11 +243,10 @@ impl Operation {
                 let event_name = event.clone();
                 let task = async move {
                     while state.count_events(&event) < count {
-                        RUNTIME.delay_for(Duration::from_millis(100)).await;
+                        runtime::delay_for(Duration::from_millis(100)).await;
                     }
                 };
-                RUNTIME
-                    .timeout(timeout.unwrap_or(EVENT_TIMEOUT), task)
+                runtime::timeout(timeout.unwrap_or(EVENT_TIMEOUT), task)
                     .await
                     .unwrap_or_else(|_| {
                         panic!("waiting for {} {} event(s) timed out", count, event_name)
@@ -321,42 +326,51 @@ impl Operation {
 }
 
 impl Matchable for TlsOptions {
-    fn content_matches(&self, expected: &TlsOptions) -> bool {
+    fn content_matches(&self, expected: &TlsOptions) -> std::result::Result<(), String> {
         self.allow_invalid_certificates
             .matches(&expected.allow_invalid_certificates)
-            && self
-                .ca_file_path
-                .as_ref()
-                .map(|pb| pb.display().to_string())
-                .matches(
-                    &expected
-                        .ca_file_path
-                        .as_ref()
-                        .map(|pb| pb.display().to_string()),
-                )
-            && self
-                .cert_key_file_path
-                .as_ref()
-                .map(|pb| pb.display().to_string())
-                .matches(
-                    &expected
-                        .cert_key_file_path
-                        .as_ref()
-                        .map(|pb| pb.display().to_string()),
-                )
+            .prefix("allow_invalid_certificates")?;
+        self.ca_file_path
+            .as_ref()
+            .map(|pb| pb.display().to_string())
+            .matches(
+                &expected
+                    .ca_file_path
+                    .as_ref()
+                    .map(|pb| pb.display().to_string()),
+            )
+            .prefix("ca_file_path")?;
+        self.cert_key_file_path
+            .as_ref()
+            .map(|pb| pb.display().to_string())
+            .matches(
+                &expected
+                    .cert_key_file_path
+                    .as_ref()
+                    .map(|pb| pb.display().to_string()),
+            )
+            .prefix("cert_key_file_path")?;
+        Ok(())
     }
 }
 
 impl Matchable for EventOptions {
-    fn content_matches(&self, expected: &EventOptions) -> bool {
-        self.max_idle_time.matches(&expected.max_idle_time)
-            && self.max_pool_size.matches(&expected.max_pool_size)
-            && self.min_pool_size.matches(&expected.min_pool_size)
+    fn content_matches(&self, expected: &EventOptions) -> std::result::Result<(), String> {
+        self.max_idle_time
+            .matches(&expected.max_idle_time)
+            .prefix("max_idle_time")?;
+        self.max_pool_size
+            .matches(&expected.max_pool_size)
+            .prefix("max_pool_size")?;
+        self.min_pool_size
+            .matches(&expected.min_pool_size)
+            .prefix("min_pool_size")?;
+        Ok(())
     }
 }
 
 impl Matchable for Event {
-    fn content_matches(&self, expected: &Event) -> bool {
+    fn content_matches(&self, expected: &Event) -> std::result::Result<(), String> {
         match (self, expected) {
             (Event::PoolCreated(actual), Event::PoolCreated(ref expected)) => {
                 actual.options.matches(&expected.options)
@@ -368,8 +382,12 @@ impl Matchable for Event {
                 actual.connection_id.matches(&expected.connection_id)
             }
             (Event::ConnectionClosed(actual), Event::ConnectionClosed(ref expected)) => {
-                actual.reason == expected.reason
-                    && actual.connection_id.matches(&expected.connection_id)
+                eq_matches("reason", &actual.reason, &expected.reason)?;
+                actual
+                    .connection_id
+                    .matches(&expected.connection_id)
+                    .prefix("connection_id")?;
+                Ok(())
             }
             (Event::ConnectionCheckedOut(actual), Event::ConnectionCheckedOut(ref expected)) => {
                 actual.connection_id.matches(&expected.connection_id)
@@ -380,12 +398,21 @@ impl Matchable for Event {
             (
                 Event::ConnectionCheckOutFailed(actual),
                 Event::ConnectionCheckOutFailed(ref expected),
-            ) => actual.reason == expected.reason,
-            (Event::ConnectionCheckOutStarted(_), Event::ConnectionCheckOutStarted(_)) => true,
-            (Event::PoolCleared(_), Event::PoolCleared(_)) => true,
-            (Event::PoolReady(_), Event::PoolReady(_)) => true,
-            (Event::PoolClosed(_), Event::PoolClosed(_)) => true,
-            _ => false,
+            ) => {
+                if actual.reason == expected.reason {
+                    Ok(())
+                } else {
+                    Err(format!(
+                        "expected reason {:?}, got {:?}",
+                        expected.reason, actual.reason
+                    ))
+                }
+            }
+            (Event::ConnectionCheckOutStarted(_), Event::ConnectionCheckOutStarted(_)) => Ok(()),
+            (Event::PoolCleared(_), Event::PoolCleared(_)) => Ok(()),
+            (Event::PoolReady(_), Event::PoolReady(_)) => Ok(()),
+            (Event::PoolClosed(_), Event::PoolClosed(_)) => Ok(()),
+            (actual, expected) => Err(format!("expected event {:?}, got {:?}", actual, expected)),
         }
     }
 }
@@ -401,12 +428,20 @@ async fn cmap_spec_tests() {
         let _guard: RwLockWriteGuard<()> = LOCK.run_exclusively().await;
 
         let mut options = CLIENT_OPTIONS.clone();
+        if options.load_balanced.unwrap_or(false) {
+            log_uncaptured(format!(
+                "skipping {:?} due to load balanced topology",
+                test_file.description
+            ));
+            return;
+        }
         options.hosts.drain(1..);
         options.direct_connection = Some(true);
         let client = EventClient::with_options(options).await;
         if let Some(ref run_on) = test_file.run_on {
             let can_run_on = run_on.iter().any(|run_on| run_on.can_run_on(&client));
             if !can_run_on {
+                log_uncaptured("skipping due to runOn requirements");
                 return;
             }
         }
