@@ -17,7 +17,13 @@ use self::options::*;
 use crate::{
     bson::{doc, to_document, Bson, Document},
     bson_util,
-    client::session::TransactionState,
+    change_stream::{
+        event::ChangeStreamEvent,
+        options::ChangeStreamOptions,
+        session::SessionChangeStream,
+        ChangeStream,
+    },
+    client::options::ServerAddress,
     cmap::conn::PinnedConnectionHandle,
     concern::{ReadConcern, WriteConcern},
     error::{convert_bulk_errors, BulkWriteError, BulkWriteFailure, Error, ErrorKind, Result},
@@ -82,7 +88,7 @@ use crate::{
 /// # #[cfg(feature = "tokio-runtime")]
 /// # use tokio::task;
 /// #
-/// # #[cfg(not(feature = "sync"))]
+/// # #[cfg(all(not(feature = "sync"), not(feature = "tokio-sync")))]
 /// # async fn start_workers() -> Result<()> {
 /// # use mongodb::Client;
 /// #
@@ -771,6 +777,7 @@ impl<T> Collection<T> {
         &self,
         cursor_id: i64,
         pinned_connection: Option<&PinnedConnectionHandle>,
+        drop_address: Option<ServerAddress>,
     ) -> Result<()> {
         let ns = self.namespace();
 
@@ -781,19 +788,68 @@ impl<T> Collection<T> {
                     "killCursors": ns.coll.as_str(),
                     "cursors": [cursor_id]
                 },
-                None,
+                drop_address.map(SelectionCriteria::from_address),
                 None,
                 pinned_connection,
             )
             .await?;
         Ok(())
     }
-}
 
-impl<T> Collection<T>
-where
-    T: DeserializeOwned + Unpin + Send + Sync,
-{
+    /// Starts a new [`ChangeStream`](change_stream/struct.ChangeStream.html) that receives events
+    /// for all changes in this collection. A
+    /// [`ChangeStream`](change_stream/struct.ChangeStream.html) cannot be started on system
+    /// collections.
+    ///
+    /// See the documentation [here](https://docs.mongodb.com/manual/changeStreams/) on change
+    /// streams.
+    ///
+    /// Change streams require either a "majority" read concern or no read concern. Anything else
+    /// will cause a server error.
+    ///
+    /// Also note that using a `$project` stage to remove any of the `_id`, `operationType` or `ns`
+    /// fields will cause an error. The driver requires these fields to support resumability. For
+    /// more information on resumability, see the documentation for
+    /// [`ChangeStream`](change_stream/struct.ChangeStream.html)
+    ///
+    /// If the pipeline alters the structure of the returned events, the parsed type will need to be
+    /// changed via [`ChangeStream::with_type`].
+    pub async fn watch(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: impl Into<Option<ChangeStreamOptions>>,
+    ) -> Result<ChangeStream<ChangeStreamEvent<T>>>
+    where
+        T: DeserializeOwned + Unpin + Send + Sync,
+    {
+        let mut options = options.into();
+        resolve_options!(self, options, [read_concern, selection_criteria]);
+        let target = self.namespace().into();
+        self.client()
+            .execute_watch(pipeline, options, target, None)
+            .await
+    }
+
+    /// Starts a new [`SessionChangeStream`] that receives events for all changes in this collection
+    /// using the provided [`ClientSession`].  See [`Client::watch`] for more information.
+    pub async fn watch_with_session(
+        &self,
+        pipeline: impl IntoIterator<Item = Document>,
+        options: impl Into<Option<ChangeStreamOptions>>,
+        session: &mut ClientSession,
+    ) -> Result<SessionChangeStream<ChangeStreamEvent<T>>>
+    where
+        T: DeserializeOwned + Unpin + Send + Sync,
+    {
+        let mut options = options.into();
+        resolve_read_concern_with_session!(self, options, Some(&mut *session))?;
+        resolve_selection_criteria_with_session!(self, options, Some(&mut *session))?;
+        let target = self.namespace().into();
+        self.client()
+            .execute_watch_with_session(pipeline, options, target, None, session)
+            .await
+    }
+
     /// Finds the documents in the collection matching `filter`.
     pub async fn find(
         &self,
@@ -803,7 +859,7 @@ where
         let mut options = options.into();
         resolve_options!(self, options, [read_concern, selection_criteria]);
 
-        let find = Find::<T>::new(self.namespace(), filter.into(), options);
+        let find = Find::new(self.namespace(), filter.into(), options);
         let client = self.client();
 
         client.execute_cursor_operation(find).await
@@ -820,12 +876,17 @@ where
         resolve_read_concern_with_session!(self, options, Some(&mut *session))?;
         resolve_selection_criteria_with_session!(self, options, Some(&mut *session))?;
 
-        let find = Find::<T>::new(self.namespace(), filter.into(), options);
+        let find = Find::new(self.namespace(), filter.into(), options);
         let client = self.client();
 
         client.execute_session_cursor_operation(find, session).await
     }
+}
 
+impl<T> Collection<T>
+where
+    T: DeserializeOwned + Unpin + Send + Sync,
+{
     /// Finds a single document in the collection matching `filter`.
     pub async fn find_one(
         &self,
@@ -1031,6 +1092,7 @@ impl<T> Collection<T>
 where
     T: Serialize,
 {
+    #[allow(clippy::needless_option_as_deref)]
     async fn insert_many_common(
         &self,
         docs: impl IntoIterator<Item = impl Borrow<T>>,
@@ -1290,7 +1352,7 @@ where
 }
 
 /// A struct modeling the canonical name for a collection in MongoDB.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct Namespace {
     /// The name of the database associated with this namespace.
     pub db: String,
@@ -1335,3 +1397,12 @@ impl<'de> Deserialize<'de> for Namespace {
         }
     }
 }
+
+impl Serialize for Namespace {
+    fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        serializer.serialize_str(&(self.db.clone() + "." + &self.coll))
+    }
+}
