@@ -46,6 +46,9 @@ pub struct Error {
     /// The type of error that occurred.
     pub kind: Box<ErrorKind>,
     labels: HashSet<String>,
+    pub(crate) wire_version: Option<i32>,
+    #[source]
+    pub(crate) source: Option<Box<Error>>,
 }
 
 impl Error {
@@ -59,6 +62,8 @@ impl Error {
         Self {
             kind: Box::new(kind),
             labels,
+            wire_version: None,
+            source: None,
         }
     }
 
@@ -235,6 +240,7 @@ impl Error {
             ErrorKind::Write(WriteFailure::WriteConcernError(wc_error)) => Some(wc_error.code),
             _ => None,
         }
+        .or_else(|| self.source.as_ref().and_then(|s| s.code()))
     }
 
     /// Gets the message for this error, if applicable, for use in testing.
@@ -315,6 +321,39 @@ impl Error {
     pub(crate) fn is_pool_cleared(&self) -> bool {
         matches!(self.kind.as_ref(), ErrorKind::ConnectionPoolCleared { .. })
     }
+
+    /// If this error is resumable as per the change streams spec.
+    pub(crate) fn is_resumable(&self) -> bool {
+        if !self.is_server_error() {
+            return true;
+        }
+        let code = self.code();
+        if code == Some(43) {
+            return true;
+        }
+        if matches!(self.wire_version, Some(v) if v >= 9)
+            && self.contains_label("ResumableChangeStreamError")
+        {
+            return true;
+        }
+        if let (Some(code), true) = (code, matches!(self.wire_version, Some(v) if v < 9)) {
+            if [
+                6, 7, 89, 91, 189, 262, 9001, 10107, 11600, 11602, 13435, 13436, 63, 150, 13388,
+                234, 133,
+            ]
+            .iter()
+            .any(|c| *c == code)
+            {
+                return true;
+            }
+        }
+        false
+    }
+
+    pub(crate) fn with_source<E: Into<Option<Error>>>(mut self, source: E) -> Self {
+        self.source = source.into().map(Box::new);
+        self
+    }
 }
 
 impl<E> From<E> for Error
@@ -338,6 +377,14 @@ impl From<bson::ser::Error> for ErrorKind {
     }
 }
 
+impl From<bson::raw::Error> for ErrorKind {
+    fn from(err: bson::raw::Error) -> Self {
+        Self::InvalidResponse {
+            message: err.to_string(),
+        }
+    }
+}
+
 impl From<std::io::Error> for ErrorKind {
     fn from(err: std::io::Error) -> Self {
         Self::Io(Arc::new(err))
@@ -432,6 +479,10 @@ pub enum ErrorKind {
     #[error("The server does not support a database operation: {message}")]
     #[non_exhaustive]
     IncompatibleServer { message: String },
+
+    /// No resume token was present in a change stream document.
+    #[error("Cannot provide resume functionality when the resume token is missing")]
+    MissingResumeToken,
 }
 
 impl ErrorKind {
