@@ -1,14 +1,14 @@
 //! This crate contains the officially supported MongoDB Rust driver, a
 //! client side library that can be used to interact with MongoDB deployments
-//! in Rust applications. It uses the [`bson`](docs.rs/bson) crate for BSON support.
-//! The driver contains a fully async API that supports either [`tokio`](docs.rs/tokio) (default)
-//! or [`async-std`](docs.rs/async-std), depending on the feature flags set. The driver also has
+//! in Rust applications. It uses the [`bson`] crate for BSON support.
+//! The driver contains a fully async API that supports either [`tokio`] (default)
+//! or [`async-std`](https://docs.rs/async_std), depending on the feature flags set. The driver also has
 //! a sync API that may be enabled via the `"sync"` feature flag.
 //!
 //! # Installation
 //!
 //! ## Requirements
-//! - Rust 1.48+
+//! - Rust 1.51+
 //! - MongoDB 3.6+
 //!
 //! ## Importing
@@ -16,7 +16,7 @@
 //! your application, simply add it to your project's `Cargo.toml`.
 //! ```toml
 //! [dependencies]
-//! mongodb = "2.1.0"
+//! mongodb = "2.2.0"
 //! ```
 //!
 //! ### Configuring the async runtime
@@ -30,7 +30,7 @@
 //! add the following to your `Cargo.toml`:
 //! ```toml
 //! [dependencies.mongodb]
-//! version = "2.1.0"
+//! version = "2.2.0"
 //! default-features = false
 //! features = ["async-std-runtime"]
 //! ```
@@ -40,7 +40,7 @@
 //! `Cargo.toml`:
 //! ```toml
 //! [dependencies.mongodb]
-//! version = "2.1.0"
+//! version = "2.2.0"
 //! default-features = false
 //! features = ["sync"]
 //! ```
@@ -50,17 +50,19 @@
 //!
 //! ### All Feature flags
 //!
-//! | Feature             | Description                                                                                                                           | Extra dependencies                  | Default |
-//! |:--------------------|:--------------------------------------------------------------------------------------------------------------------------------------|:------------------------------------|:--------|
-//! | `tokio-runtime`     | Enable support for the `tokio` async runtime                                                                                          | `tokio` 1.0 with the `full` feature | yes     |
-//! | `async-std-runtime` | Enable support for the `async-std` runtime                                                                                            | `async-std` 1.0                     | no      |
-//! | `sync`              | Expose the synchronous API (`mongodb::sync`). This flag cannot be used in conjunction with either of the async runtime feature flags. | `async-std` 1.0                     | no      |
-//! | `aws-auth`          | Enable support for the MONGODB-AWS authentication mechanism.                                                                          | `reqwest` 0.11                      | no      |
-//! | `bson-uuid-0_8`     | Enable support for v0.8 of the [`uuid`](docs.rs/uuid/0.8) crate in the public API of the re-exported `bson` crate.                    | n/a                                 | no      |
-//! | `bson-chrono-0_4`   | Enable support for v0.4 of the [`chrono`](docs.rs/chrono/0.4) crate in the public API of the re-exported `bson` crate.                | n/a                                 | no      |
-//! | `zlib-compression`]  | Enable support for compressing messages with [`zlib`](https://zlib.net/).                                                            | `flate2` 1.0                        | no      |
-//! | `zstd-compression`  | Enable support for compressing messages with [`zstd`](http://facebook.github.io/zstd/).  This flag requires Rust version 1.54.        | `zstd` 0.9.0                        | no      |
-//! | `snappy-compression` | Enable support for compressing messages with [`snappy`](http://google.github.io/snappy/).                                            | `snap` 1.0.5                        | no      |
+//! | Feature              | Description                                                                                                                           | Extra dependencies                  | Default |
+//! |:---------------------|:--------------------------------------------------------------------------------------------------------------------------------------|:------------------------------------|:--------|
+//! | `tokio-runtime`      | Enable support for the `tokio` async runtime                                                                                          | `tokio` 1.0 with the `full` feature | yes     |
+//! | `async-std-runtime`  | Enable support for the `async-std` runtime                                                                                            | `async-std` 1.0                     | no      |
+//! | `sync`               | Expose the synchronous API (`mongodb::sync`). This flag cannot be used in conjunction with either of the async runtime feature flags. | `async-std` 1.0                     | no      |
+//! | `aws-auth`           | Enable support for the MONGODB-AWS authentication mechanism.                                                                          | `reqwest` 0.11                      | no      |
+//! | `bson-uuid-0_8`      | Enable support for v0.8 of the [`uuid`](docs.rs/uuid/0.8) crate in the public API of the re-exported `bson` crate.                    | n/a                                 | no      |
+//! | `bson-chrono-0_4`    | Enable support for v0.4 of the [`chrono`](docs.rs/chrono/0.4) crate in the public API of the re-exported `bson` crate.                | n/a                                 | no      |
+//! | `bson-serde_with`    | Enable support for the [`serde_with`](docs.rs/serde_with/latest) crate in the public API of the re-exported `bson` crate.             | `serde_with` 1.0                    | no      |
+//! | `zlib-compression`   | Enable support for compressing messages with [`zlib`](https://zlib.net/)                                                              | `flate2` 1.0                        | no      |
+//! | `zstd-compression`   | Enable support for compressing messages with [`zstd`](http://facebook.github.io/zstd/).  This flag requires Rust version 1.54.        | `zstd` 0.9.0                        | no      |
+//! | `snappy-compression` | Enable support for compressing messages with [`snappy`](http://google.github.io/snappy/)                                              | `snap` 1.0.5                        | no      |
+//! | `openssl-tls`        | Switch TLS connection handling to use ['openssl'](https://docs.rs/openssl/0.10.38/).                                                  | `openssl` 0.10.38                   | no      |
 //!
 //! # Example Usage
 //!
@@ -200,7 +202,7 @@
 //! crate's top level like in the async API. The sync API calls through to the async API internally
 //! though, so it looks and behaves similarly to it.
 //! ```no_run
-//! # #[cfg(feature = "sync")]
+//! # #[cfg(any(feature = "sync", feature = "tokio-sync"))]
 //! # {
 //! use mongodb::{
 //!     bson::doc,
@@ -265,7 +267,7 @@
 //! #     bson::doc,
 //! # };
 //! #
-//! # #[cfg(all(not(feature = "sync"), feature = "tokio-runtime"))]
+//! # #[cfg(all(not(feature = "sync"), not(feature = "tokio-sync"), feature = "tokio-runtime"))]
 //! # async fn foo() -> std::result::Result<(), Box<dyn std::error::Error>> {
 //! #
 //! # let client = Client::with_uri_str("mongodb://example.com").await?;
@@ -281,7 +283,7 @@
 //!
 //! ## Minimum supported Rust version (MSRV)
 //!
-//! The MSRV for this crate is currently 1.48.0. This will be rarely be increased, and if it ever is,
+//! The MSRV for this crate is currently 1.51.0. This will be rarely be increased, and if it ever is,
 //! it will only happen in a minor or major version release.
 
 #![warn(missing_docs)]
@@ -299,7 +301,7 @@
 )]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![cfg_attr(test, type_length_limit = "80000000")]
-#![doc(html_root_url = "https://docs.rs/mongodb/2.1.0")]
+#![doc(html_root_url = "https://docs.rs/mongodb/2.2.0")]
 
 #[cfg(all(feature = "aws-auth", feature = "async-std-runtime"))]
 compile_error!("The `aws-auth` feature flag is only supported on the tokio runtime.");
@@ -310,6 +312,7 @@ pub mod options;
 pub use ::bson;
 
 mod bson_util;
+pub mod change_stream;
 mod client;
 mod cmap;
 mod coll;
@@ -321,15 +324,15 @@ mod db;
 pub mod error;
 pub mod event;
 mod index;
-mod is_master;
+mod hello;
 mod operation;
 pub mod results;
 pub(crate) mod runtime;
 mod sdam;
 mod selection_criteria;
 mod srv;
-#[cfg(any(feature = "sync", docsrs))]
-#[cfg_attr(docsrs, doc(cfg(feature = "sync")))]
+#[cfg(any(feature = "sync", feature = "tokio-sync", docsrs))]
+#[cfg_attr(docsrs, doc(cfg(any(feature = "sync", feature = "tokio-sync"))))]
 pub mod sync;
 #[cfg(test)]
 mod test;
@@ -338,7 +341,7 @@ mod test;
 #[macro_use]
 extern crate derive_more;
 
-#[cfg(not(feature = "sync"))]
+#[cfg(all(not(feature = "sync"), not(feature = "tokio-sync")))]
 pub use crate::{
     client::{Client, session::ClientSession},
     coll::Collection,
@@ -346,7 +349,7 @@ pub use crate::{
     db::Database,
 };
 
-#[cfg(feature = "sync")]
+#[cfg(any(feature = "sync", feature = "tokio-sync"))]
 pub(crate) use crate::{
     client::{Client, session::ClientSession},
     coll::Collection,
@@ -359,7 +362,8 @@ pub use {coll::Namespace, index::IndexModel, client::session::ClusterTime, sdam:
 #[cfg(all(
     feature = "tokio-runtime",
     feature = "async-std-runtime",
-    not(feature = "sync")
+    not(feature = "sync"),
+    not(feature = "tokio-sync"),
 ))]
 compile_error!(
     "`tokio-runtime` and `async-std-runtime` can't both be enabled; either disable \
@@ -368,18 +372,21 @@ compile_error!(
 
 #[cfg(all(feature = "tokio-runtime", feature = "sync"))]
 compile_error!(
-    "`tokio-runtime` and `sync` can't both be enabled; either disable `sync` or set \
-     `default-features = false` in your Cargo.toml"
+    "`tokio-runtime` and `sync` can't both be enabled; either disable `sync` and enable \
+     `tokio-sync` to use the sync API with tokio, or set `default-features = false` in \
+     your Cargo.toml to use the sync API with async-std"
+);
+
+#[cfg(all(feature = "async-std-runtime", feature = "tokio-sync"))]
+compile_error!(
+    "`async-std-runtime` and `tokio-sync` can't both be enabled; either disable `tokio-sync` \
+     and enable `sync` to use the sync API with async-std, or disable `async-std-runtime` to \
+     use the sync API with tokio"
 );
 
 #[cfg(all(not(feature = "tokio-runtime"), not(feature = "async-std-runtime")))]
 compile_error!(
-    "one of `tokio-runtime`, `async-std-runtime`, or `sync` must be enabled; either enable \
-     `default-features`, or enable one of those features specifically in your Cargo.toml"
+    "one of `tokio-runtime`, `async-std-runtime`, `sync`, or `tokio-sync` must be enabled; \
+     either enable `default-features`, or enable one of those features specifically in your \
+     Cargo.toml"
 );
-
-#[cfg(all(feature = "tokio-runtime", not(feature = "async-std-runtime")))]
-pub(crate) static RUNTIME: runtime::AsyncRuntime = runtime::AsyncRuntime::Tokio;
-
-#[cfg(all(not(feature = "tokio-runtime"), feature = "async-std-runtime"))]
-pub(crate) static RUNTIME: runtime::AsyncRuntime = runtime::AsyncRuntime::AsyncStd;
