@@ -1,18 +1,22 @@
+mod change_stream;
+
 #[cfg(test)]
 mod test;
 
 use crate::{
     bson::{doc, Bson, Document},
     bson_util,
-    cmap::{Command, StreamDescription},
+    cmap::{Command, RawCommandResponse, StreamDescription},
     cursor::CursorSpecification,
     error::Result,
-    operation::{append_options, Operation, Retryability},
+    operation::{append_options, remove_empty_write_concern, Operation, Retryability},
     options::{AggregateOptions, SelectionCriteria, WriteConcern},
     Namespace,
 };
 
-use super::{CursorBody, CursorResponse};
+use super::{CursorBody, WriteConcernOnlyBody, SERVER_4_2_0_WIRE_VERSION};
+
+pub(crate) use change_stream::ChangeStreamAggregate;
 
 #[derive(Debug)]
 pub(crate) struct Aggregate {
@@ -40,10 +44,11 @@ impl Aggregate {
     }
 }
 
+// IMPORTANT: If new method implementations are added here, make sure `ChangeStreamAggregate` has
+// the equivalent delegations.
 impl Operation for Aggregate {
-    type O = CursorSpecification<Document>;
+    type O = CursorSpecification;
     type Command = Document;
-    type Response = CursorResponse<Document>;
 
     const NAME: &'static str = "aggregate";
 
@@ -53,6 +58,8 @@ impl Operation for Aggregate {
             "pipeline": bson_util::to_bson_array(&self.pipeline),
             "cursor": {}
         };
+
+        remove_empty_write_concern!(self.options);
         append_options(&mut body, self.options.as_ref())?;
 
         if self.is_out_or_merge() {
@@ -61,24 +68,35 @@ impl Operation for Aggregate {
             }
         }
 
-        Ok(Command::new(
+        Ok(Command::new_read(
             Self::NAME.to_string(),
             self.target.db_name().to_string(),
+            self.options.as_ref().and_then(|o| o.read_concern.clone()),
             body,
         ))
     }
 
+    fn extract_at_cluster_time(
+        &self,
+        response: &bson::RawDocument,
+    ) -> Result<Option<bson::Timestamp>> {
+        CursorBody::extract_at_cluster_time(response)
+    }
+
     fn handle_response(
         &self,
-        response: CursorBody<Document>,
+        response: RawCommandResponse,
         description: &StreamDescription,
     ) -> Result<Self::O> {
+        let cursor_response: CursorBody = response.body()?;
+
         if self.is_out_or_merge() {
-            response.write_concern_info.validate()?;
+            let wc_error_info = response.body::<WriteConcernOnlyBody>()?;
+            wc_error_info.validate()?;
         };
 
         Ok(CursorSpecification::new(
-            response.cursor,
+            cursor_response.cursor,
             description.server_address.clone(),
             self.options.as_ref().and_then(|opts| opts.batch_size),
             self.options.as_ref().and_then(|opts| opts.max_await_time),
@@ -91,6 +109,12 @@ impl Operation for Aggregate {
             .and_then(|opts| opts.selection_criteria.as_ref())
     }
 
+    fn supports_read_concern(&self, description: &StreamDescription) -> bool {
+        // for aggregates that write, read concern is only supported in MongoDB 4.2+.
+        !self.is_out_or_merge()
+            || description.max_wire_version.unwrap_or(0) >= SERVER_4_2_0_WIRE_VERSION
+    }
+
     fn write_concern(&self) -> Option<&WriteConcern> {
         self.options
             .as_ref()
