@@ -1,11 +1,13 @@
 #[cfg(test)]
 mod test;
 
+use bson::Document;
+
 use crate::{
     bson::doc,
-    cmap::{Command, CommandResponse, StreamDescription},
+    cmap::{Command, RawCommandResponse, StreamDescription},
     error::Result,
-    operation::{append_options, Operation, WriteConcernOnlyBody},
+    operation::{append_options, remove_empty_write_concern, Operation, WriteConcernOnlyBody},
     options::{CreateCollectionOptions, WriteConcern},
     Namespace,
 };
@@ -35,12 +37,16 @@ impl Create {
 
 impl Operation for Create {
     type O = ();
+    type Command = Document;
+
     const NAME: &'static str = "create";
 
-    fn build(&self, _description: &StreamDescription) -> Result<Command> {
+    fn build(&mut self, _description: &StreamDescription) -> Result<Command> {
         let mut body = doc! {
             Self::NAME: self.ns.coll.clone(),
         };
+
+        remove_empty_write_concern!(self.options);
         append_options(&mut body, self.options.as_ref())?;
 
         Ok(Command::new(
@@ -50,8 +56,13 @@ impl Operation for Create {
         ))
     }
 
-    fn handle_response(&self, response: CommandResponse) -> Result<Self::O> {
-        response.body::<WriteConcernOnlyBody>()?.validate()
+    fn handle_response(
+        &self,
+        response: RawCommandResponse,
+        _description: &StreamDescription,
+    ) -> Result<Self::O> {
+        let response: WriteConcernOnlyBody = response.body()?;
+        response.validate()
     }
 
     fn write_concern(&self) -> Option<&WriteConcern> {
