@@ -100,7 +100,7 @@ pub(crate) struct TopologyDescription {
     /// acceptable for a read operation.
     pub(crate) local_threshold: Option<Duration>,
 
-    /// The maximum amount of time to wait before checking a given server by sending an isMaster.
+    /// The maximum amount of time to wait before checking a given server by sending server check.
     pub(crate) heartbeat_freq: Option<Duration>,
 
     /// The server descriptions of each member of the topology.
@@ -140,15 +140,7 @@ impl TopologyDescription {
         let servers: HashMap<_, _> = options
             .hosts
             .into_iter()
-            .map(|address| {
-                let description = if topology_type == TopologyType::LoadBalanced {
-                    ServerDescription::new_load_balancer(address.clone())
-                } else {
-                    ServerDescription::new(address.clone(), None)
-                };
-
-                (address, description)
-            })
+            .map(|address| (address.clone(), ServerDescription::new(address, None)))
             .collect();
 
         let session_support_status = if topology_type == TopologyType::LoadBalanced {
@@ -158,6 +150,11 @@ impl TopologyDescription {
         } else {
             SessionSupportStatus::Undetermined
         };
+        let transaction_support_status = if topology_type == TopologyType::LoadBalanced {
+            TransactionSupportStatus::Supported
+        } else {
+            TransactionSupportStatus::Undetermined
+        };
 
         Ok(Self {
             single_seed: servers.len() == 1,
@@ -167,7 +164,7 @@ impl TopologyDescription {
             max_election_id: None,
             compatibility_error: None,
             session_support_status,
-            transaction_support_status: TransactionSupportStatus::Undetermined,
+            transaction_support_status,
             cluster_time: None,
             local_threshold: options.local_threshold,
             heartbeat_freq: options.heartbeat_freq,
@@ -264,12 +261,8 @@ impl TopologyDescription {
         match read_preference {
             ReadPreference::Secondary { .. }
             | ReadPreference::PrimaryPreferred { .. }
-            | ReadPreference::Nearest { .. } => {
-                command.set_read_preference(read_preference.clone())
-            }
-            ReadPreference::SecondaryPreferred { ref options }
-                if options.max_staleness.is_some() || options.tag_sets.is_some() =>
-            {
+            | ReadPreference::Nearest { .. }
+            | ReadPreference::SecondaryPreferred { .. } => {
                 command.set_read_preference(read_preference.clone())
             }
             _ => {}
@@ -474,6 +467,12 @@ impl TopologyDescription {
             server_description.clone(),
         );
 
+        if let TopologyType::LoadBalanced = self.topology_type {
+            // Load-balanced topologies don't have real server updates; attempting to update based
+            // on the synthesized one causes incorrect behavior.
+            return Ok(());
+        }
+
         // Update the topology's min logicalSessionTimeout.
         self.update_session_support_status(&server_description);
 
@@ -696,7 +695,7 @@ impl TopologyDescription {
         let addresses: Vec<_> = self.servers.keys().cloned().collect();
 
         // If any other servers are RSPrimary, replace them with an unknown server decscription,
-        // which will cause them to be updated by a new isMaster.
+        // which will cause them to be updated by a new server check.
         for address in addresses.clone() {
             if address == server_description.address {
                 continue;
