@@ -5,11 +5,15 @@ use std::{
 
 use bson::{bson, doc};
 use semver::VersionReq;
-use tokio::sync::RwLockWriteGuard;
+use tokio::sync::{RwLockReadGuard, RwLockWriteGuard};
 
 use crate::{
     error::ErrorKind,
+    hello::{LEGACY_HELLO_COMMAND_NAME, LEGACY_HELLO_COMMAND_NAME_LOWERCASE},
+    runtime,
+    sdam::ServerType,
     test::{
+        log_uncaptured,
         CmapEvent,
         Event,
         EventClient,
@@ -23,7 +27,6 @@ use crate::{
         LOCK,
     },
     Client,
-    RUNTIME,
 };
 
 #[cfg_attr(feature = "tokio-runtime", tokio::test(flavor = "multi_thread"))]
@@ -32,18 +35,26 @@ async fn min_heartbeat_frequency() {
     let _guard: RwLockWriteGuard<_> = LOCK.run_exclusively().await;
 
     let mut setup_client_options = CLIENT_OPTIONS.clone();
+    if setup_client_options.load_balanced.unwrap_or(false) {
+        log_uncaptured("skipping min_heartbeat_frequency test due to load-balanced topology");
+        return;
+    }
     setup_client_options.hosts.drain(1..);
     setup_client_options.direct_connection = Some(true);
 
     let setup_client = TestClient::with_options(Some(setup_client_options.clone())).await;
 
-    if !setup_client.supports_fail_command().await {
-        println!("skipping min_heartbeat_frequency test due to server not supporting fail points");
+    if !setup_client.supports_fail_command() {
+        log_uncaptured(
+            "skipping min_heartbeat_frequency test due to server not supporting fail points",
+        );
         return;
     }
 
     if setup_client.server_version_lt(4, 9) {
-        println!("skipping min_heartbeat_frequency test due to server version being less than 4.9");
+        log_uncaptured(
+            "skipping min_heartbeat_frequency test due to server version being less than 4.9",
+        );
         return;
     }
 
@@ -51,8 +62,11 @@ async fn min_heartbeat_frequency() {
         .app_name("SDAMMinHeartbeatFrequencyTest".to_string())
         .error_code(1234)
         .build();
-    let failpoint =
-        FailPoint::fail_command(&["isMaster", "hello"], FailPointMode::Times(5), fp_options);
+    let failpoint = FailPoint::fail_command(
+        &[LEGACY_HELLO_COMMAND_NAME, "hello"],
+        FailPointMode::Times(5),
+        fp_options,
+    );
 
     let _fp_guard = setup_client
         .enable_failpoint(failpoint, None)
@@ -90,6 +104,10 @@ async fn sdam_pool_management() {
     let _guard: RwLockWriteGuard<_> = LOCK.run_exclusively().await;
 
     let mut options = CLIENT_OPTIONS.clone();
+    if options.load_balanced.unwrap_or(false) {
+        log_uncaptured("skipping sdam_pool_management test due to load-balanced topology");
+        return;
+    }
     options.hosts.drain(1..);
     options.direct_connection = Some(true);
     options.app_name = Some("SDAMPoolManagementTest".to_string());
@@ -110,8 +128,8 @@ async fn sdam_pool_management() {
         .unwrap()
         .matches(&client.server_version)
     {
-        println!(
-            "skipping sdam_pool_management test due to server not supporting appName failCommand"
+        log_uncaptured(
+            "skipping sdam_pool_management test due to server not supporting appName failCommand",
         );
         return;
     }
@@ -134,8 +152,11 @@ async fn sdam_pool_management() {
         .app_name("SDAMPoolManagementTest".to_string())
         .error_code(1234)
         .build();
-    let failpoint =
-        FailPoint::fail_command(&["isMaster", "hello"], FailPointMode::Times(1), fp_options);
+    let failpoint = FailPoint::fail_command(
+        &[LEGACY_HELLO_COMMAND_NAME, "hello"],
+        FailPointMode::Times(1),
+        fp_options,
+    );
 
     let _fp_guard = client
         .enable_failpoint(failpoint, None)
@@ -179,6 +200,10 @@ async fn sdam_min_pool_size_error() {
     let _guard: RwLockWriteGuard<_> = LOCK.run_exclusively().await;
 
     let mut setup_client_options = CLIENT_OPTIONS.clone();
+    if setup_client_options.load_balanced.unwrap_or(false) {
+        log_uncaptured("skipping sdam_min_pool_size_error test due to load-balanced topology");
+        return;
+    }
     setup_client_options.hosts.drain(1..);
     setup_client_options.direct_connection = Some(true);
 
@@ -188,8 +213,8 @@ async fn sdam_min_pool_size_error() {
         .unwrap()
         .matches(&setup_client.server_version)
     {
-        println!(
-            "skipping sdam_pool_management test due to server not supporting appName failCommand"
+        log_uncaptured(
+            "skipping sdam_pool_management test due to server not supporting appName failCommand",
         );
         return;
     }
@@ -198,8 +223,11 @@ async fn sdam_min_pool_size_error() {
         .app_name("SDAMMinPoolSizeErrorTest".to_string())
         .error_code(1234)
         .build();
-    let failpoint =
-        FailPoint::fail_command(&["isMaster", "hello"], FailPointMode::Skip(3), fp_options);
+    let failpoint = FailPoint::fail_command(
+        &[LEGACY_HELLO_COMMAND_NAME, "hello"],
+        FailPointMode::Skip(3),
+        fp_options,
+    );
 
     let _fp_guard = setup_client
         .enable_failpoint(failpoint, None)
@@ -227,7 +255,7 @@ async fn sdam_min_pool_size_error() {
     // Wait a little while for the server to be marked as Unknown.
     // Once we have SDAM monitoring, this wait can be removed and be replaced
     // with another event waiting.
-    RUNTIME.delay_for(Duration::from_millis(750)).await;
+    runtime::delay_for(Duration::from_millis(750)).await;
 
     let ping_err = client
         .database("admin")
@@ -271,12 +299,12 @@ async fn auth_error() {
         .unwrap()
         .matches(&setup_client.server_version)
     {
-        println!("skipping auth_error test due to server not supporting appName failCommand");
+        log_uncaptured("skipping auth_error test due to server not supporting appName failCommand");
         return;
     }
 
     if !setup_client.auth_enabled() {
-        println!("skipping auth_error test due to auth not being enabled");
+        log_uncaptured("skipping auth_error test due to auth not being enabled");
         return;
     }
 
@@ -306,6 +334,7 @@ async fn auth_error() {
     options.retry_writes = Some(false);
     options.cmap_event_handler = Some(handler.clone());
     options.command_event_handler = Some(handler.clone());
+    options.sdam_event_handler = Some(handler.clone());
     let client = Client::with_options(options).expect("client creation should succeed");
 
     let coll = client.database("auth_error").collection("auth_error");
@@ -315,6 +344,18 @@ async fn auth_error() {
         .expect_err("insert should fail");
     assert!(matches!(*auth_err.kind, ErrorKind::Authentication { .. }));
 
+    if !setup_client.is_load_balanced() {
+        subscriber
+            .wait_for_event(Duration::from_millis(2000), |event| match event {
+                Event::Sdam(SdamEvent::ServerDescriptionChanged(event)) => {
+                    event.new_description.description.server_type == ServerType::Unknown
+                }
+                _ => false,
+            })
+            .await
+            .expect("should see server marked unknown");
+    }
+
     subscriber
         .wait_for_event(Duration::from_millis(2000), |event| {
             matches!(event, Event::Cmap(CmapEvent::PoolCleared(_)))
@@ -322,11 +363,6 @@ async fn auth_error() {
         .await
         .expect("should see pool cleared event");
 
-    // Wait a little while for the server to be marked as Unknown.
-    // Once we have SDAM monitoring, this wait can be removed and be replaced
-    // with another event waiting.
-    RUNTIME.delay_for(Duration::from_millis(750)).await;
-
     coll.insert_many(vec![doc! { "_id": 5 }, doc! { "_id": 6 }], None)
         .await
         .expect("insert should succeed");
@@ -349,3 +385,69 @@ async fn auth_error() {
         &vec![bson!({ "_id": 5 }), bson!({ "_id": 6 })]
     );
 }
+
+#[cfg_attr(feature = "tokio-runtime", tokio::test(flavor = "multi_thread"))]
+#[cfg_attr(feature = "async-std-runtime", async_std::test)]
+async fn hello_ok_true() {
+    let _guard: RwLockReadGuard<_> = LOCK.run_concurrently().await;
+
+    let mut setup_client_options = CLIENT_OPTIONS.clone();
+    setup_client_options.hosts.drain(1..);
+
+    if setup_client_options.server_api.is_some() {
+        log_uncaptured("skipping hello_ok_true test due to ServerApi being configured");
+        return;
+    }
+
+    if setup_client_options.load_balanced == Some(true) {
+        log_uncaptured("skipping hello_ok_true test due to load balanced topology");
+        return;
+    }
+
+    let setup_client = TestClient::with_options(Some(setup_client_options.clone())).await;
+    if !VersionReq::parse(">= 4.4.5")
+        .unwrap()
+        .matches(&setup_client.server_version)
+    {
+        log_uncaptured("skipping hello_ok_true test due to server not supporting hello");
+        return;
+    }
+
+    let handler = Arc::new(EventHandler::new());
+    let mut subscriber = handler.subscribe();
+
+    let mut options = setup_client_options.clone();
+    options.sdam_event_handler = Some(handler.clone());
+    options.direct_connection = Some(true);
+    options.heartbeat_freq = Some(Duration::from_millis(500));
+    let _client = Client::with_options(options).expect("client creation should succeed");
+
+    // first heartbeat should be legacy hello but contain helloOk
+    subscriber
+        .wait_for_event(Duration::from_millis(2000), |event| {
+            if let Event::Sdam(SdamEvent::ServerHeartbeatSucceeded(e)) = event {
+                assert_eq!(e.reply.get_bool("helloOk"), Ok(true));
+                assert!(e.reply.get(LEGACY_HELLO_COMMAND_NAME_LOWERCASE).is_some());
+                assert!(e.reply.get("isWritablePrimary").is_none());
+                return true;
+            }
+            false
+        })
+        .await
+        .expect("first heartbeat reply should contain helloOk: true");
+
+    // subsequent heartbeats should just be hello
+    for _ in 0..3 {
+        subscriber
+            .wait_for_event(Duration::from_millis(2000), |event| {
+                if let Event::Sdam(SdamEvent::ServerHeartbeatSucceeded(e)) = event {
+                    assert!(e.reply.get("isWritablePrimary").is_some());
+                    assert!(e.reply.get(LEGACY_HELLO_COMMAND_NAME_LOWERCASE).is_none());
+                    return true;
+                }
+                false
+            })
+            .await
+            .expect("subsequent heartbeats should use hello");
+    }
+}
