@@ -155,7 +155,7 @@ impl<'de> Deserialize<'de> for ReadPreference {
             "PrimaryPreferred" => Ok(ReadPreference::PrimaryPreferred {
                 options: preference.options,
             }),
-            "SecondaryPreferred" => Ok(ReadPreference::SecondaryPreferred {
+            "SecondaryPreferred" | "secondaryPreferred" => Ok(ReadPreference::SecondaryPreferred {
                 options: preference.options,
             }),
             "Nearest" => Ok(ReadPreference::Nearest {
@@ -323,8 +323,6 @@ impl ReadPreference {
     where
         S: serde::Serializer,
     {
-        use serde::ser::Serialize;
-
         #[derive(serde::Serialize)]
         struct ReadPreferenceHelper<'a> {
             readpreference: &'a str,
